// VFD Clock for ATMEGA88P
// 2012/01/15 Eleken

// main clock : 8.0 MHz

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

//#include "main.h"

  uint8_t FONTS[11] = {0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x27, 0x7F, 0x6F, 0x40}; // JP Font
// const uint8_t FONTS[11] = {0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7C, 0x07, 0x7F, 0x67, 0x40}; // US Font

// definition of constants
#define TIMER0_T0 255-120
#define TIMER0_CONST 30

#define PWM_TOP 160
#define PWM_MAX 128

#define UART_BAUD 9600

// declare variables

	volatile uint8_t digit[10], cursel[10];
	uint8_t  cur_time[7]; // s,m,h,w,d,m,y; s=cur_time[0]
	volatile uint8_t sw_state; // state of switch [0-4] : none, A, B, C, A-long
	volatile uint8_t ui_state; // state of display [255, 0-3]
	volatile uint8_t adj_state; // in adjusting?

	volatile uint8_t brightness; // (dark) [0-3] (bright)

	volatile uint8_t measure_temp_flag;
	int16_t temperature;
	uint8_t blink;
	int16_t temp_today_max, temp_today_min;

	volatile uint8_t hold_disp_cnt; // display is holded while this variable > 0

// recent max/min temperature
// these variables are placed in eeprom
// 64 * (2+2+1+1) + 2 = 386 [bytes]
	uint8_t dummy __attribute__((section(".eeprom"))) = 0;
	uint8_t temp_pointer __attribute__((section(".eeprom"))) = 0;
	uint8_t temp_saved_cnt __attribute__((section(".eeprom"))) = 0;
	uint16_t temp_recent_max[64] __attribute__((section(".eeprom")));
	uint16_t temp_recent_min[64] __attribute__((section(".eeprom")));
	uint8_t temp_recent_month[64] __attribute__((section(".eeprom")));
	uint8_t temp_recent_day[64] __attribute__((section(".eeprom")));


// declare functions

	void decimal_add(uint8_t* p);
	void decimal_dec(uint8_t* p);
	void measure_temp(void);
	static void UART_init(void);
	static void init(void);
	void eep_write_temp(uint8_t month, uint8_t day, int16_t max, int16_t min);
	uint8_t eep_read_temp(uint8_t days_ago, uint8_t* month, uint8_t* day, int16_t* max, int16_t* min);


// timer1 : 19us
ISR(TIMER1_OVF_vect){
	// control DC/DC
	uint16_t pwm = OCR1B;
	if(ACSR & _BV(ACO)){ // 1.1[V] > VFB
		if(pwm < PWM_MAX) pwm ++;
	}else{
		if(pwm >= 1) pwm --;
	}
	
	OCR1B = pwm;
	
}

// timer0 : 100us
ISR(TIMER0_OVF_vect){
	static uint8_t cnt_60hz;
	
	static uint8_t t0cnt = 0;
	static uint8_t sw_old, longpush_cnt;
	static uint8_t digitsel = 0;
	
//	sei(); // allow interrupt
	
	// output dc-ac inverter
	if(cnt_60hz++ > 30){
		cnt_60hz = 0;
		if((PORTD & 0b00011000) == 0b00010000) PORTD = (PORTD & 0b11100111) | 0b00001000;
		else PORTD = (PORTD & 0b11100111) | 0b00010000;
	}
	
	if(t0cnt == 0){ // check switch state ( 220us * 200 = 44ms )
		uint8_t sw = (PIND & 0x60);
		
		if(sw_old == sw){// valid only when switch was pushed.
			sw_state = 0;// switch state has no change
		}else {
			if(sw == 0x20){//B
				sw_state = 2;
			}else if(sw == 0x40){//C
				sw_state = 3;
			}else if(sw == 0x60){//off
				if(longpush_cnt == 0) sw_state = 0; // longpush?
				else if(longpush_cnt <= 14) sw_state = 1; // < 616ms
				else sw_state = 4;
			}else sw_state = 0;
		}
		if(sw == 0x00){ //Both
			longpush_cnt ++;
		}else{
			longpush_cnt = 0;
		}
		sw_old = sw;
	}
	
	// segment clear
	PORTB &= 0b11000111;
	PORTC &= 0b11100001;
	PORTD &= 0b11111001;
	_delay_us(40); // wait 20us
	
	
	// grid shift
	if(digitsel == 0) PORTB |= _BV(PB0);
	else PORTB &= ~_BV(PB0);
	__asm__ volatile ("nop");
	PORTB |=  _BV(PB1);
	__asm__ volatile ("nop");
	PORTB &= ~_BV(PB1);
	_delay_us(40); // wait 20us
	uint8_t i;
	for(i=0; i<3-brightness; i++) _delay_us(TIMER0_CONST); // wait (brightness adjust)

	// segment show
	PORTB |= (digit[digitsel] & 0x07) << 3;
	PORTC |= ((digit[digitsel] >> 3) & 0x0F) << 1;
	if(digit[digitsel] & 0x80) PORTD |= _BV(PD1);;
	if(cursel[digitsel]) PORTD |= _BV(PD2);
	
	t0cnt ++;
	if(t0cnt >= 200) t0cnt = 0;
	digitsel ++;
	if(digitsel >= 10) digitsel = 0;
	
	TCNT0 = 255 - TIMER0_CONST*(brightness+1);
}

// timer2 : 2hz
ISR(TIMER2_OVF_vect){
//	PORTC = PORTC ^ 0b00010000;
	
	if((blink & 0x01) && (adj_state == 0)) {
		decimal_add(&cur_time[0]); // sec
		if(cur_time[0] >= 0x60){
			cur_time[0] = 0;
			decimal_add(&cur_time[1]); // min
			if(cur_time[1] >= 0x60){
				cur_time[1] = 0;
				decimal_add(&cur_time[2]); // hour
				if(cur_time[2] >= 0x24){ // increase current day
					uint8_t day_max = 0x31; // max of day in this month & year
					if((cur_time[5]==0x04) || (cur_time[5]==0x06) || (cur_time[5]==0x11)) day_max = 0x30;
					else if(cur_time[5] == 0x02){
						if(((cur_time[6]>>4)*10+(cur_time[6]&0x0F)) % 0x04 == 0) day_max = 0x29; // is uruu year?
						else day_max = 0x28;
					}
					cur_time[2] = 0;
					
					// temperature
				//	eep_write_temp(cur_time[5], cur_time[4], temp_today_max, temp_today_min); // save yesterday's statistics
				//	temp_today_max = temp_today_min = temperature; // reset statistics
					
					decimal_add(&cur_time[4]); // day
					if(cur_time[4] > day_max){
						cur_time[4] = 1;
						decimal_add(&cur_time[5]); // month
						if(cur_time[5] > 0x12){
							cur_time[5] = 1;
							decimal_add(&cur_time[6]); // year
						}
					}
				}
			}
		}
		
		
	}else{
		if((PINC & 0b00100000) != 0) { // Vcc = H
			measure_temp_flag = 1;
			if(hold_disp_cnt) hold_disp_cnt--;
		}
	}
	blink++;
}

// USART receive
ISR(USART_RX_vect){
	static uint8_t received_data[20];
	static uint8_t state; // {0,1,2}
	static uint8_t dcnt;
	uint8_t d = UDR0;
	uint8_t i;
	
	if(state == 0){
		if(d == 0x5A){
			state = 1;
			dcnt = 0;
		}else if(d == 0x5B){
			state = 2;
			dcnt = 0;
		}
	}else if(state == 1){
		received_data[dcnt++] = d;
		if(dcnt == 10) { // finished
			state = 0;
			for(i=0; i<10; i++) digit[i] = received_data[i];
			ui_state = 255;
		}
	}else if(state == 2){
		received_data[dcnt++] = d;
		if(dcnt == 20) { // finished
			state = 0;
			for(i=0; i<10; i++) digit[i] = received_data[i];
			for(i=0; i<10; i++) cursel[i] = received_data[i+10];
			ui_state = 255;
		}
	}
}

// if main power is disconnected, then sleep
void check_power(void){
	if((PINC & 0b00100000) != 0) return; // Vcc = H
	uint8_t PB_old = PORTB, PC_old = PORTC, PD_old = PORTD;
	uint8_t DB_old = DDRB, DC_old = DDRC, DD_old = DDRD;
	PORTB = PORTC = PORTD = 0;
	DDRB = DDRC = DDRD = 0;
	
	while(1){
		MCUCR = 0b01100000;
		MCUCR = 0b01000000;
		PRR   = 0b10101111; // disable peripheral
		ACSR  = 0b11000000; // disable comparator
		sei();
		SMCR = 0b00000111; // power save / sleep enable 
		__asm__ volatile ("sleep");
		SMCR = 0b00000000; // sleep diasvle
		
		if((PINC & 0b00100000) != 0) { // Vcc = H
			// Peripheral
			MCUCR = 0b00000000;
			PRR   = 0; // enable peripheral
			ACSR  = 0b01000000; // ACBG = 1
			break;
		}
	}
	
	PORTB = PB_old;
	PORTC = PC_old;
	PORTD = PD_old;
	DDRB = DB_old;
	DDRC = DC_old;
	DDRD = DD_old;
	UART_init();
}

void decimal_add(uint8_t* p){
	if(((*p) & 0x0F) == 9) (*p) += 7;
	else (*p) ++;
}
void decimal_dec(uint8_t* p){
	if(((*p) & 0x0F) == 0) (*p) -= 7;
	else (*p) --;
}

// measure temperature
#define TEMP_AVG_WIDTH 16 // must be 2^N
void measure_temp(void){
	static int16_t temp_old[TEMP_AVG_WIDTH];
	static uint8_t to_cnt;
	int16_t adc_res = 0;
	uint8_t i;
	for(i=0; i<10; i++){
		ADCSRA = 0b10010110; //CK/64
		ADCSRA = 0b11000110; //CK/64
		while((ADCSRA & _BV(ADIF)) == 0);
		adc_res += ADC - 246; // 0-1024
	}

	// scaling
	// for LM61 (V = 600 + 10*T[mV])
	// Vref = 2.5[V] (0-1024 -> 0-2.5v)
//	adc_res *= 10;
	adc_res /= 4;

	// filtering
	if(to_cnt) temp_old[to_cnt++ & (TEMP_AVG_WIDTH-1)] = adc_res;
	else {// first time
		for(i=0; i<TEMP_AVG_WIDTH; i++){
			temp_old[i] =  adc_res;
		}
		to_cnt++;
	}
	if(to_cnt > TEMP_AVG_WIDTH*2) to_cnt -= TEMP_AVG_WIDTH;
	adc_res = 0;
	for(i=0; i<TEMP_AVG_WIDTH; i++){
		adc_res += temp_old[i];
	}
	adc_res /= TEMP_AVG_WIDTH;
	
	// hex -> dec
	temperature = 0;
	while(adc_res >= 100) {
		temperature += 0x0100;
		adc_res -= 100;
	}
	while(adc_res >= 10) {
		temperature += 0x0010;
		adc_res -= 10;
	}
	while(adc_res <= -100) {
		temperature -= 0x0100;
		adc_res += 100;
	}
	while(adc_res <= -10) {
		temperature -= 0x0010;
		adc_res += 10;
	}
	temperature += adc_res;
	
	if(temp_today_min == 0){
		temp_today_min = temp_today_max = temperature;
	}
	if(temperature < temp_today_min) temp_today_min = temperature;
	if(temperature > temp_today_max) temp_today_max = temperature;
}

// push to eeprom temperature of yesterday
void eep_write_temp(uint8_t month, uint8_t day, int16_t max, int16_t min){
	// vfd off
	uint8_t i;
	
	// TIMER
	TCCR1A = 0b00000011; // disconnect OC1B(PB2)
	PORTB &= ~_BV(PB2);
	
	
	// 
	PORTB &= ~_BV(PB0);
	for(i=0; i<8; i++){
		__asm__ volatile ("nop");
		PORTB |=  _BV(PB1);
		__asm__ volatile ("nop");
		PORTB &= ~_BV(PB1);
	}
	
	// eep write
	uint8_t epointer   = eeprom_read_byte(&temp_pointer);
	uint8_t saved_cnt = eeprom_read_byte(&temp_saved_cnt) + 1;
	
	eeprom_write_word( &(temp_recent_max[epointer]), 16384 + max);
	eeprom_write_word( &(temp_recent_min[epointer]), 16384 + min);
	eeprom_write_byte( &(temp_recent_month[epointer]), month);
	eeprom_write_byte( &(temp_recent_day[epointer]), day);
	temp_recent_min[epointer] = min;
	
	epointer = (epointer + 1) & 63;
	eeprom_write_byte( &temp_pointer, epointer);
	eeprom_write_byte( &temp_saved_cnt, saved_cnt);
	
	// Reset TIMER1
	TCCR1A = 0b00100011; // output PWM to OC1B, enable high-speed PWM
	OCR1B  = 2; // PWM
	TCNT1  = 0;
	TIMSK1 = 1; // enable overflow interrput
}
// read temperature from eeprom
uint8_t eep_read_temp(uint8_t days_ago, uint8_t* month, uint8_t* day, int16_t* max, int16_t* min){
	
	// segment clear
	PORTB &= 0b11000111;
	PORTC &= 0b11100001;
	PORTD &= 0b11111001;
	uint8_t epointer   = eeprom_read_byte(&temp_pointer);
	uint8_t saved_cnt = eeprom_read_byte(&temp_saved_cnt);
	if(saved_cnt == 0) return 1;
	else if(saved_cnt < days_ago) days_ago = saved_cnt;
	
	epointer = (epointer - days_ago) & 63;
	*max = eeprom_read_word( &(temp_recent_max[epointer])) - 16384;
	*min = eeprom_read_word( &(temp_recent_min[epointer])) - 16384;
	*month = eeprom_read_byte( &(temp_recent_month[epointer]));
	*day = eeprom_read_byte( &(temp_recent_day[epointer]));
	return 0;
}

void UART_init(void){
	// UART
	UBRR0  = (8000000 / 16 / UART_BAUD) - 1;
	UCSR0A = 0;
	UCSR0B = 0b10010000; // Allow receive, enable interrupt
	UCSR0C = 0b00000110;
	
}
void init(void){
	
	cli();
	// init clock
	CLKPR = 0x80;
	CLKPR = 0x00; // CLKPS = 0 (8MHz)
	
	MCUSR = 0;
	
	// init pins
	PORTB = 0b00000000;
	DDRB  = 0b00111111;
	
	PORTC = 0b00000000;
	DDRC  = 0b00011110;
	
	PORTD = 0b01100001;
	DDRD  = 0b00011110;
	
/*	// Vcc = L when started
	if((PINC & 0b00100000) == 0) { // Vcc = L
		while(1){
			SMCR   = 0b00000101;
			WDTCSR = 0b00110001;
			WDTCSR = 0b00111001; // enable watchdog (reset, 8s)
			__asm__ volatile ("sleep"); // sleep forever!
		}
	}
*/
	
	// init IO registers
	
	// TIMER
	TCCR1A = 0b00100011; // output PWM to OC0B, enable high-speed PWM
	TCCR1B = 0b00011001; // CS0 = 1 (CLK/1)
	OCR1A  = PWM_TOP; // TOP = 160
	OCR1B  = 2; // PWM
	TCNT1  = 0;
	TIMSK1 = 1; // enable overflow interrput
	
	TCCR0A = 0;
	TCCR0B = 0b00000010; // CS1 = 2 (CLK/8)
	TCNT0  = TIMER0_T0;
	TIMSK0 = 1; // enable overflow interrupt
	TIMSK0 = 1;
	
	ASSR   = 0b00100000; // Enable asynchronous timer (32khz)
	TCCR2A = 0;
	TCCR2B = 0b00000100; // CS2 = 4 (CLK/64)
	TCNT2  = 0;
	TIMSK2 = 1; // enable overflow interrupt
	
	
	// External interrput 
//	PCICR  = 0b00000010; // enable pin-change interrput 1
//	PCMSK1 = 0b00100000; // PCINT13 (PC5)
	
	// Analog comparator
	ACSR  = 0b01000000; // ACBG = 1
	DIDR1 = 0b00000010; // Disable PD7 (AIN1)
	
	// ADC
	DIDR0 = 0b00000001; // Disable PC0
	ADMUX = 0b00000000;
	ADCSRA = 0b10000110; //CK/64
	ADCSRB = 0;
	
	UART_init();
	
	uint8_t i;
	for(i=0; i<7; i++) cur_time[i] = 0;
	for(i=0; i<10; i++) digit[i] = FONTS[i];
	for(i=0; i<10; i++) cursel[i] = 0;
	brightness = 3;
	
	sei();
}


int main(void){
	init();
	
	uint8_t i;
	for(i=0; i<100; i++) _delay_ms(10.0); // wait 1.0[s]
//	rtc_read();
	uint8_t days_ago = 0;
	int16_t display_temp_max, display_temp_min;

	
	while(1){
		uint8_t latched_sw_state = sw_state;
		sw_state = 0;
		if(adj_state == 0){
			if(latched_sw_state == 1){ // push
				days_ago = 0;
				ui_state ++;
				if(ui_state == 4){ // Temperature (Today's max & min)
					hold_disp_cnt = 1;
					display_temp_max = temp_today_max;
					display_temp_min = temp_today_min;
					
					for(i=0; i<6; i++) digit[i] = 0;
					digit[9] = 0b01110110; // H
					digit[8] = 0b00000110; // I
					digit[7] = 0b00111101; // G
					digit[6] = 0b01110110; // H
					digit[4] = 0b00111000; // L
					digit[3] = 0b01011100; // o
				}
				if(ui_state > 4) ui_state = 0;
			}else if(latched_sw_state == 4){ // longpush
				adj_state = 1;
				ui_state  = 3;
				cur_time[0] = 0;
			}else if(latched_sw_state == 2){ // +
				if(ui_state == 4){ // Temperature
					if(days_ago > 1) days_ago--;
					uint8_t month, day;
					if(eep_read_temp(days_ago, &month, &day, &display_temp_max, &display_temp_min) == 0){
						hold_disp_cnt = 2;
						for(i=0; i<6; i++) digit[i] = 0;
						digit[9] = FONTS[(month>>4)];
						digit[8] = FONTS[(month&0x0F)] | 0x80;
						digit[7] = FONTS[(day>>4)];
						digit[6] = FONTS[(day&0x0F)];
					}
				}else{
					if(brightness < 3) brightness ++;
				}
			}else if(latched_sw_state == 3){ // -
				if(ui_state == 4){ // Temperature
					days_ago++;
					uint8_t month, day;
					if(eep_read_temp(days_ago, &month, &day, &display_temp_max, &display_temp_min) == 0){
						hold_disp_cnt = 2;
						for(i=0; i<6; i++) digit[i] = 0;
						digit[9] = FONTS[(month>>4)];
						digit[8] = FONTS[(month&0x0F)] | 0x80;
						digit[7] = FONTS[(day>>4)];
						digit[6] = FONTS[(day&0x0F)];
					}
				}else{
					if(brightness > 0) brightness --;
				}
			}
		}else{
			if(latched_sw_state == 1){
				adj_state ++;
				if(adj_state == 5) ui_state = 0;
				else if(adj_state == 8) adj_state = 0;
			}
			else if(latched_sw_state == 2){ // +
				switch (adj_state){
				case 1:
					cur_time[6] += 0x10;
					break;
				case 2:
					decimal_add(&cur_time[6]);
					break;
				case 3:
					decimal_add(&cur_time[5]);
					break;
				case 4:
					decimal_add(&cur_time[4]);
					break;
				case 5:
					decimal_add(&cur_time[2]);
					break;
				case 6:
					cur_time[1] += 0x10;
					break;
				case 7:
					if((cur_time[1] & 0x0F) == 9) cur_time[1] -= 0x09;
					else cur_time[1] += 0x01;
//					decimal_add(&cur_time[1]);
					break;
				default:
					break;
				}
			}
			else if(latched_sw_state == 3){ // -
				switch (adj_state){
				case 1:
					cur_time[6] -= 0x10;
					break;
				case 2:
					decimal_dec(&cur_time[6]);
					break;
				case 3:
					decimal_dec(&cur_time[5]);
					break;
				case 4:
					decimal_dec(&cur_time[4]);
					break;
				case 5:
					decimal_dec(&cur_time[2]);
					break;
				case 6:
					cur_time[1] -= 0x10;
					break;
				case 7:
					if((cur_time[1] & 0x0F) == 0) cur_time[1] += 0x09;
					else cur_time[1] -= 0x01;
//					decimal_dec(&cur_time[1]);
					break;
				default:
					break;
				}
			}
			if(cur_time[6] >= 0xF0) cur_time[6] = 0x90 + (cur_time[6]&0x0F);
			else if(cur_time[6] > 0x99) cur_time[6] &= 0x0F;
			if(cur_time[5] == 0) cur_time[5] = 0x12;
			else if(cur_time[5] > 0x12) cur_time[5] = 1;
			if(cur_time[4] == 0) cur_time[4] = 0x31;
			else if(cur_time[4] > 0x31) cur_time[4] = 1;
			if(cur_time[2] == 0xF9) cur_time[2] = 0x23;
			else if(cur_time[2] > 0x23) cur_time[2] = 0;
			if(cur_time[1] >= 0xF0) cur_time[1] = 0x50 + (cur_time[1]&0x0F);
			else if(cur_time[1] > 0x59) cur_time[1] = 0;
			if(cur_time[0] >= 0xF0) cur_time[0] = 0x50 + (cur_time[0]&0x0F);
			else if(cur_time[0] > 0x59) cur_time[0] = 0;
		}
		
		// cursel
		{
		uint8_t t = 1; // month: 1-9
		if(cur_time[5]&0xF0) t = 0; // month: 10-12
		cli(); // avoid cursel flicking
		for(i=0; i<10; i++) cursel[i] = 0;
		switch (adj_state){
		case 1:
			cursel[t?6:7] = 1;
			break;
		case 2:
			cursel[t?5:6] = 1;
			break;
		case 3:
			cursel[4] = 1;
			cursel[3] = 1;
			break;
		case 4:
			cursel[1] = 1;
			cursel[0] = 1;
			break;
		case 5:
			cursel[7] = 1;
			cursel[8] = 1;
			break;
		case 6:
			cursel[5] = 1;
			break;
		case 7:
			cursel[4] = 1;
			break;
		case 8:
			cursel[2] = 1;
			break;
		case 9:
			cursel[1] = 1;
			break;
		default:
			break;
		}
		sei();
		}
		
		if(!hold_disp_cnt){
			switch (ui_state){
			int8_t dgcnt;
			uint16_t t;
			case 1:
				// "hh.mm xx.x`C"
				cli();
				digit[9] = (cur_time[2]&0xF0) ? FONTS[cur_time[2]>>4]: 0;
				digit[8] = FONTS[cur_time[2]&0x0F];
				if(blink & 0x01) digit[8] |= 0x80;
				digit[7] = FONTS[cur_time[1]>>4];
				digit[6] = FONTS[cur_time[1]&0x0F];
				if(temperature >= 0){
					t = temperature;
					digit[5] = 0;
					digit[4] = (t & 0x0F00) ? FONTS[(t>>8) & 0x0F]: 0;
				}else{
					t = -temperature;
					if(t & 0x0F00){
						digit[5] = FONTS[10];
						digit[4] = FONTS[(t>>8) & 0x0F];
					}else{
						digit[5] = 0;
						digit[4] = FONTS[10];
					}
				}
				digit[3] = FONTS[(t>>4) & 0x0F] | 0x80;
				digit[2] = FONTS[t & 0x0F];
				digit[1] = 0b01100011;
				digit[0] = 0b00111001;
				sei();
				break;
			case 2:
				// "hh.mm.ss xx.x"
				dgcnt = 9;
				if(cur_time[2]&0xF0) digit[dgcnt--] = FONTS[cur_time[2]>>4];
				digit[dgcnt--] = FONTS[cur_time[2]&0x0F] | 0x80;
				digit[dgcnt--] = FONTS[cur_time[1]>>4];
				digit[dgcnt--] = FONTS[cur_time[1]&0x0F] | 0x80;
				digit[dgcnt--] = FONTS[cur_time[0]>>4];
				digit[dgcnt--] = FONTS[cur_time[0]&0x0F];
				if(temperature >= 0){
					t = temperature;
					digit[dgcnt--] = 0;
				}else{
					t = -temperature;
					digit[dgcnt--] = FONTS[10];
				}
				if(t & 0x0F00) digit[dgcnt--] = FONTS[(t>>8) & 0x0F];
				digit[dgcnt--] = FONTS[(t>>4) & 0x0F] | 0x80;
				digit[dgcnt--] = FONTS[t & 0x0F];
				if(dgcnt >= 0) {
					cli();
					digit[0] = 0;
					digit[dgcnt] = 0b01011000; // c
					sei();
				}
				break;
			case 3:
				// " 20yy.mm.dd. "
				{
					uint8_t t = 1;
					if(cur_time[5]&0xF0) t = 0;
					digit[9-t] = FONTS[2];
					digit[8-t] = FONTS[0];
					digit[7-t] = FONTS[cur_time[6]>>4];
					digit[6-t] = FONTS[cur_time[6]&0x0F] | 0x80;
					digit[t?9:5] = 0;
					digit[4] = t ? 0: FONTS[(cur_time[5])>>4];
				}
				digit[3] = FONTS[(cur_time[5])&0x0F] | 0x80;
				digit[2] = 0;
				digit[1] = FONTS[(cur_time[4])>>4];
				digit[0] = FONTS[(cur_time[4])&0x0F] | 0x80;
				break;
			case 4:
				// "xx.x'cxx.x'c"
				dgcnt = 9;
				if(display_temp_max >= 0){
					t = display_temp_max;
				}else{
					t = -display_temp_max;
					digit[dgcnt--] = FONTS[10];
				}
				if(t & 0x0FF0) digit[dgcnt--] = FONTS[(t>>8) & 0x0F];
				digit[dgcnt--] = FONTS[(t>>4) & 0x0F] | 0x80;
				digit[dgcnt--] = FONTS[t & 0x0F];
				digit[dgcnt--] = 0b01100011;
				digit[dgcnt--] = 0b00111001;
				
				if(display_temp_min >= 0){
					t = display_temp_min;
				}else{
					t = -display_temp_min;
					digit[dgcnt--] = FONTS[10];
				}
				if(t & 0x0F00) digit[dgcnt--] = FONTS[(t>>8) & 0x0F];
				else if(dgcnt == 4) digit[dgcnt--] = 0;
				digit[dgcnt--] = FONTS[(t>>4) & 0x0F] | 0x80;
				digit[dgcnt--] = FONTS[t & 0x0F];
				if(dgcnt >= 0) {
					digit[dgcnt--] = 0b01100011;
					digit[dgcnt] = 0b00111001;
				}else if(dgcnt == 0) {
					digit[dgcnt] = 0b01011000; // c
				}
				break;
			case 255:
				break;
			default:
				// " hh-mm-ss "
				digit[9] = 0;
				digit[8] = FONTS[cur_time[2]>>4];
				digit[7] = FONTS[cur_time[2]&0x0F];
				digit[6] = FONTS[10];
				digit[5] = FONTS[cur_time[1]>>4];
				digit[4] = FONTS[cur_time[1]&0x0F];
				digit[3] = FONTS[10];
				digit[2] = FONTS[cur_time[0]>>4];
				digit[1] = FONTS[cur_time[0]&0x0F];
				digit[0] = 0;
				break;
			}
		}
		// Check Power
		check_power();
		
		// Measure temperature
		if(measure_temp_flag){
			measure_temp();
			measure_temp_flag = 0;
		}
	}

	return 0;
}
